@@ -12,12 +12,11 @@ import json
 import hashlib
 
 try:
-    import urllib3
-    pool = urllib3.PoolManager()
-except:
-    pool = None
+    import urllib2
+except ImportError:
+    import urllib.request as urllib2
 
-__version__ = "0.1.18"
+__version__ = "0.1.19"
 __author__ = "Xiaocong He"
 __all__ = ["device", "Device", "rect", "point", "Selector"]
 
@@ -54,10 +53,27 @@ def param_to_property(*props, **kwprops):
     return Wrapper
 
 
+class _ConnectionPool(object):
+
+    def __init__(self):
+        self.count = 0
+        self.pool = None
+
+    def __get__(self, obj, cls):
+        if self.count == 0 or self.pool == None:
+            # reset connection pool to workaround NanoHttpd overflow exception
+            import urllib3
+            self.pool = urllib3.PoolManager(1)
+        self.count = (self.count + 1) % 32
+        return self.pool
+
+
 class JsonRPCMethod(object):
 
+    pool = _ConnectionPool()
+
     def __init__(self, url, method, timeout=30):
-        self.url, self.method, self.timeout, self.pool = url, method, timeout, pool
+        self.url, self.method, self.timeout = url, method, timeout
 
     def __call__(self, *args, **kwargs):
         if args and kwargs:
@@ -67,10 +83,23 @@ class JsonRPCMethod(object):
             data["params"] = args
         elif kwargs:
             data["params"] = kwargs
-        res = self.pool.urlopen("POST", self.url, headers={'Content-Type':'application/json'}, body=json.dumps(data).encode("utf-8"))
-        if res is None or res.status != 200:
-            raise Exception("Error reponse from jsonrpc server.")
-        jsonresult = json.loads(res.data.decode("utf-8"))
+        if os.name == "nt":
+            res = self.pool.urlopen("POST",
+                                    self.url,
+                                    headers={"Content-Type": "application/json"},
+                                    body=json.dumps(data).encode("utf-8"),
+                                    timeout=self.timeout)
+            if res is None or res.status != 200:
+                raise Exception("Error reponse from jsonrpc server.")
+            jsonresult = json.loads(res.data.decode("utf-8"))
+        else:
+            req = urllib2.Request(self.url,
+                                  json.dumps(data).encode("utf-8"),
+                                  {"Content-type": "application/json"})
+            result = urllib2.urlopen(req, timeout=self.timeout)
+            if result is None or result.getcode() != 200:
+                raise Exception("Error reponse from jsonrpc server.")
+            jsonresult = json.loads(result.read().decode("utf-8"))
         if "error" in jsonresult and jsonresult["error"]:
             raise Exception("Error response. Error code: %d, Error message: %s" %
                             (jsonresult["error"]["code"], jsonresult["error"]["message"]))
@@ -278,7 +307,6 @@ class AutomatorServer(object):
     }
 
     def __init__(self, serial=None, local_port=None):
-        self.pool = pool
         self.uiautomator_process = None
         self.adb = Adb(serial=serial)
         self.device_port = 9008
@@ -308,7 +336,7 @@ class AutomatorServer(object):
 
     def download(self, filename, url):
         with open(filename, 'wb') as file:
-            file.write(self.pool.urlopen("GET", url).data)
+            file.write(urllib2.urlopen(url).read())
 
     @property
     def jsonrpc(self):
@@ -354,7 +382,7 @@ class AutomatorServer(object):
         '''Stop the rpc server.'''
         if self.uiautomator_process and self.uiautomator_process.poll() is None:
             try:
-                self.pool.urlopen("GET", self.stop_uri)
+                urllib2.urlopen(self.stop_uri)
                 self.uiautomator_process.wait()
             except:
                 self.uiautomator_process.kill()
